<?php

namespace Lean\Api;

use Lean\Factory\HttpFactory;
use Notes\ObjectReflection;
use Notes\Route\Attribute\Method\Route;
use Picea\Extension\UrlExtension;

class RouteDescriptor
{
    public function __construct(
        public object $controller,
        protected UrlExtension $urlExtension,
    ) {}

    public function getRoutes() : array
    {
        $routes = [];

        $reflector = new ObjectReflection($this->controller::class);

        $attribute = $reflector->reflectClass()->getAttribute(\Notes\Route\Attribute\Object\Route::class);

        $base = $attribute ? $attribute->object->base : "";

        foreach($reflector->reflectMethods() as $method) {
            foreach(array_reverse($method->getAttributes(Route::class)) as $routeAttribute) {
                $route = $routeAttribute->object;
                $path = rtrim($route->route, '/');
                $cleaned = $this->cleanRouteFromRegex($base.$path);
                $url = $this->urlExtension->buildUrl($cleaned);

                $routes[] = [
                    'name' => $route->name,
                    'route' => $url,
                    'path' => $base.$path,
                    'cleaned' => $cleaned,
                    'description'=> $route->description,
                    'methods' => $route->method,
                ];
            }
        }

        return $routes;
    }

    protected function cleanRouteFromRegex(string $route) : string
    {
        $paths = explode('/', $route);

        foreach($paths as &$path) {
            list($newPath, ) = explode(':', $path);

            $path = $newPath === $path ? $path : "$newPath}";
        }

        return implode('/', $paths);
    }
}